<?php

require_once "config.inc";
require_once "privpref.inc";
require_once "constants.inc";
require_once "language.inc";

if (defined('USER_TEMPLATE')) {
	$layout_config = config_to_array('templates/' . USER_TEMPLATE . '/layout_config');
} else {
	$layout_config = config_to_array('templates/default/layout_config');
}

function replace_ascii(&$string)
{
	return (
		str_replace (" ", "%20",
		str_replace ("", "%e5",
		str_replace ("", "%e4",
		str_replace ("", "%f6",
		str_replace ("", "%c5",
		str_replace ("", "%c4",
		str_replace ("", "%d6",
		str_replace ("&aring;", "",
		str_replace ("&auml;", "",
		str_replace ("&ouml;", "",
		str_replace ("&Aring;", "",
		str_replace ("&Auml;", "",
		str_replace ("&Ouml;", "",
		$string
		)))))))))))))
	);
}

function umlaut_str(&$string)
{
	return (
		str_replace ("&aring;", "",
		str_replace ("&auml;", "",
		str_replace ("&ouml;", "",
		str_replace ("&Aring;", "",
		str_replace ("&Auml;", "",
		str_replace ("&Ouml;", "",
		$string
		))))))
	);
}

function str_umlaut(&$string)
{
	return (
		str_replace ("", "&aring;",
		str_replace ("", "&auml;",
		str_replace ("", "&ouml;",
		str_replace ("", "&Aring;",
		str_replace ("", "&Auml;",
		str_replace ("", "&Ouml;",
		$string
		))))))
	);
}

//<-----
//	string layout_read_template(string file)
//
//	Reads a template file specified as a parameter
//	and removes lines beginning with #. File content
//	is returned.
//----->
function layout_read_template($file)
{
	$content = "";

	$filename = "templates/" . USER_TEMPLATE . "/" . $file;

	if (!file_exists($filename)) {
		$filename = "templates/default/" . $file;

		if (!file_exists($filename)) {
			die ("Could not find layout file $file!");
		}
	}

	$filecontent = file($filename, 1);
	foreach($filecontent as $line)
	{
		if(substr($line, 0, 1) == "#")
			continue;

		$content .= $line;
	}

	return $content;
}

//<-----
//	layout_print_header:	Print the header HTML code.
//							Optional parameters are given in an array to the function.
//----->
function layout_print_header($args = array())
{
	global	$page_config,
			$layout_config,
			$PageID;

	$variables = get_page_config($args);

	if ($variables['onload'] != '') {
		$onload = ' onLoad="' . $variables['onload'] . '"';
	}
	else {
		$onload = '';
	}

	$stylesheet_filename = 'templates/' . USER_TEMPLATE . '/' . $variables['stylesheet'];

	if (!file_exists($stylesheet_filename)) {
		$stylesheet_filename = 'templates/default/' . $variables['stylesheet'];

		if (!file_exists($stylesheet_filename)) {
			die ("Could not find stylesheet " . $variables['stylesheet']);
		}
	}

	if ($stylesheet_filename != 'templates/default/style.css')
	{
		$stylesheet = "/* Default style rules */" . "\n\n";
		$stylesheet .= join('', file('templates/default/style.css'));
		$stylesheet .= "\n\n\n" . "/* Style rules specific for " . USER_TEMPLATE . " */" . "\n\n";
		$stylesheet .= join('', file($stylesheet_filename));
	}
	else
	{
		$stylesheet = join('', file($stylesheet_filename));
	}

	$content = str_replace('%title%', $variables['title'], layout_read_template($layout_config['header']['filename']));
	$content = str_replace('%onload%', $onload, $content);
	$content = str_replace('%stylesheet%', $stylesheet, $content);
	//$content = str_replace('%stylesheet%', 'templates/' . USER_TEMPLATE . '/' . $stylesheet, $content);

	$javascript = '';

	if (is_array($variables['scripts'])) {

		foreach ($variables['scripts'] as $src)
		{
			$javascript .= eval(join('', file('js/' . $src)));
			//$javascript .= "<script type='text/javascript' src='lib/$src' defer='defer'></script>\n"; //defer = script does not modify page content
		}

	}
	$content = str_replace('%javascript%', $javascript, $content);

	$img_template_path = 'templates/' . USER_TEMPLATE . '/img/';
	$content = str_replace('%img_template_path%', $img_template_path, $content);

	print $content;
}

function convert_submenu_lang($menu_text)
{
	global $langText;

	if (is_array($menu_text))
	{
		foreach ($menu_text as $key => $element)
		{
			$menu_text[$key] = $langText['SUBMENU_' . strtoupper($element)];
		}
	}
	else {
		$menu_text = $langText['SUBMENU_' . strtoupper($menu_text)];
	}

	return $menu_text;
}

//<-----
//
//----->
function get_page_section($page_id = '')
{
	global	$PageID,
			$site_config;

	if ($page_id == '') {
		$page_id = $PageID;
	}

	foreach ($site_config as $section => $page_ids)
	{
		if (is_array($page_ids))
		{
			if (array_search($page_id, $page_ids)) {
				return $section;
			}
		}
	}
	
	return '';
}

//<----
//
//---->
function get_section_home($section)
{
	global $page_config, $site_config;

	if (!isset($site_config[$section])) {
		return '';
	}

	if (has_page_priv($site_config[$section]['home']))
	{
		return $site_config[$section]['home'];
	}
	else
	{
		foreach ($site_config[$section] as $page_ident)
		{
			$pageconf = get_page_config(array(), $page_ident);

			if (has_page_priv($page_ident) && $pageconf['menu_target'] == '_top')
			{
				return $page_ident;
			}
		}
	}

	return ''; //if nothing has been returned before this, no page has privs
}

//<----
//
//---->
function layout_get_location()
{
	global	$PageID,
			$langText,
			$page_config;

	$location = '&nbsp;&gt;&nbsp;<a href="' . $page_config[$PageID]['href']
				. '?id=' . $_REQUEST['id'] . '&auth=' . $_REQUEST['auth']
				. '">'
				. $langText['SUBMENU_' . strToUpper($PageID)]
				. '</a>';

	$section = get_page_section($PageID);

	while (!is_numeric($section) && $section != '')
	{
		$location = '&nbsp;&gt;&nbsp;<a href="' . $page_config[$section]['href']
					. '?id=' . $_REQUEST['id'] . '&auth=' . $_REQUEST['auth']
					. '">'
					. $langText['SUBMENU_' . strToUpper($section)]
					. '</a>'
					. $location;

		$section = get_page_section($section);
	}

	$section_home = get_section_home($section); //as $section now is a numeric main section

	if ($section_home == '') {
		return 0; //an error has occured
	}

	$location = '<a href="' . $page_config[$section_home]['href']
				. '?id=' . $_REQUEST['id'] . '&auth=' . $_REQUEST['auth']
				. '">'
				. $langText['MENUBUTTON_' . $section]
				. '</a>'
				. $location;

	return $langText['LOCATION'] . ':&nbsp;&nbsp;' . $location;
}

//<-----
//	int count_submenu_items(int $group)
//
//	Returns the number of visible submenu entries for a specific group in site_config.
//----->
function count_submenu_items($group)
{
	global $site_config;

	$count = 0;

	if (is_array($site_config[$group]))
	{
		foreach($site_config[$group] as $key => $value)
		{
			if (is_numeric($key)) {
				$count++;
			}
		}
	}

	return $count;
}

//<-----
//	layout_get_submenu:		Generate arrays required by layout_print_submenu() by checking
//							which main category the $PageID belongs to and thus including
//							all page information from $page_config for the pages in that
//							category.
//----->
function layout_get_submenu($page)
{
	global	$page_config,
			$site_config;

	$menu_hrefs = array();
	$menu_texts = array();
	$menu_targets = array();

	$section = get_page_section($page);

	if ($section == '') {
		$section = 1;
	}
	elseif (!is_numeric($section)) {
		return layout_get_submenu($section);
		//if this is a subsection of another page, it will have the same submenu as that page
	}

	foreach ($site_config[$section] as $subnumber => $page_id)
	{
		if (count_submenu_items($section) <= 1) {
			break;
			//if there are none or just one item, no submenu is printed!
			//checked again at end of this function
		}
		elseif ($subnumber == 'home') { //&& $page_id != $site_config[$number]['home'])
			continue;
		}
		elseif (!is_numeric($subnumber)) {
			continue;
		}
		else
		{
			$variables = get_page_config(array(), $page_id);

			if (has_page_priv($page_id))
			{
				array_push($menu_hrefs, $variables['href']);
				array_push($menu_targets, $variables['menu_target']);

				if ($variables['menu_text'] == $page_id) {
					//a PageID is a call to a language entry: this is usually the case
					array_push($menu_texts, convert_submenu_lang($variables['menu_text']));
				}
				else {
					array_push($menu_texts, $variables['menu_text']);
				}
			}
		}
	}

	//one item in category == the startpage, no submenu entry
	if (sizeof($menu_hrefs) == 1)
	{
		$menu_hrefs = array();
		$menu_texts = array();
		$menu_targets = array();
	}

	return array(
				'menu_href' => $menu_hrefs,
				'menu_text' => $menu_texts,
				'menu_target' => $menu_targets
				);
}

//<-----
//	layout_print_submenu:	Prints the submenu. Takes one parameter which is an array including
//							the names of the menus.
//----->
function layout_print_submenu($menu_href = array(), $menu_text = array(), $menu_target = array())
{
	global $layout_config;

	$menu_link = array();
	$menu_img_text = array();

	if (is_array($menu_href))
	{
		foreach($menu_href as $current)
		{
			array_push($menu_link, "$current?id=" . $_REQUEST['id'] . "&auth=" . $_REQUEST['auth']);
		}
	}

	if (is_array($menu_text))
	{
		foreach($menu_text as $current)
		{
			$current = str_replace(" ", "_", $current);
			array_push($menu_img_text, $current);
		}
	}

	for ($i = 0; $i < 6; $i++) //6 is the max number of submenu items!
	{
		if ($i < sizeof($menu_href) && is_array($menu_href))
		{
			$insert_target = ($i < sizeof($menu_target) && is_array($menu_target) && $menu_target[$i] != '') ? " target='" . $menu_target[$i] . "'" : "";

			$submenu = layout_read_template($layout_config['submenu']['active_filename']);
			$submenu = str_replace('%href%', $menu_link[$i], $submenu);
			$submenu = str_replace('%text%', $menu_text[$i], $submenu);
			$submenu = str_replace('%target%', $insert_target, $submenu);

			$img_src = 'submenu_img.php?template=' . USER_TEMPLATE . '&text=' . replace_ascii($menu_img_text[$i]);
			$submenu = str_replace('%img_src%', $img_src, $submenu);

			echo $submenu;
		}
		else
		{
			if (array_key_exists('empty_filename', $layout_config['submenu']))
			{
				$submenu = str_replace("%img_template_path%", "templates/" . USER_TEMPLATE . "/img/", layout_read_template($layout_config['submenu']['empty_filename']));
				echo $submenu;
			}
		}
	}
}

//<-----
//	layout_part_tag_replace:	Replaces vital parts of the different part-includes
//								with the correct content.
//----->
function layout_part_tag_replace($button, &$content, $priv)
{
	global	$langText,
			$config,
			$page_config;

	$home = get_section_home($button);
	$href = ($home != '') ? $page_config[$home]['href'] : '';

	if($priv && $href != '') //check twice: href is empty if no page has priv
	{
		$mouseOverText = umlaut_str($langText[strToUpper("MENUBUTTON_" . $button)]);

		$content = str_replace("%button_${button}_href%", "$href?id=" . $_REQUEST['id'] . "&auth=" . $_REQUEST['auth'], $content);
		$content = str_replace("%button_${button}_mouseover%", " onMouseOver=\"window.status='$mouseOverText'; return true\" onMouseOut=\"window.status=''; return true\"", $content);
		$content = str_replace("%button_${button}_img%", "button_$button.png", $content);
		$content = str_replace("%button_${button}_alt%", $mouseOverText, $content);
		$content = str_replace("%button_${button}_width%", layout_imgW("button_$button.png"), $content);
	}
	else
	{
		$nonoText = umlaut_str($langText["MENUBUTTON_NONO"]);

		$content = str_replace("%button_${button}_href%", "javascript:alert('$nonoText');", $content);
		$content = str_replace("%button_${button}_mouseover%", " onMouseOver=\"window.status=''; return true\"", $content);
		$content = str_replace("%button_${button}_img%", "button_${button}_nono.png", $content);
		$content = str_replace("%button_${button}_alt%", "", $content);
	}

	if(isset($config) && isset($config['bugtracker']) && $config['bugtracker'] == 1 && strstr($content, '%bugtracker%'))
	{
		global $config;
		require_once "search.inc";

		$request = '';
		foreach ($_REQUEST as $key => $value) {
			$request .= $key . ' = ' . $value . "; ";
		}

		$user_realname = USER_SURNAME . ', ' . USER_GIVEN_NAMES;

		$in_groups = '';
		$manager_of = '';
		$query = mysql_query("SELECT * from groups, linkage WHERE groups.group_id=linkage.group_id AND linkage.user_id=" . USER_ID);
		while ($row = mysql_fetch_array($query)) {

			$in_groups .= $row['group_id'] . ':' . $row['group_name'] . "; ";

			if ($row['group_manager'] == 1) {
				$manager_of .= $row['group_id'] . ':' . $row['group_name'] . "; ";
			}
		}

		$privs_array = array("ENABLE", "BORROW", "RETURN", "BORROWED_BOOKS", "ADD", "EDIT", "DELETE", "GROUP_MANAGEMENT");
		$privs = '';
		foreach ($privs_array as $priv) {
			if (check_priv($priv)) {
				$privs .= $priv . '; ';
			}
		}

		$borrowed = 0;
		$expired = 0;
		$query = mysql_query("SELECT *, UNIX_TIMESTAMP(borrowed_date) AS borrowed_epoch from borrowed WHERE borrowed_user_id=" . USER_ID);
		while ($row = mysql_fetch_array($query)) {

			$borrowed++;

			list($formatted, $epoch) = expiry_date($row['borrowed_epoch']);
			if ($epoch < time()) {
				$expired++;
			}
		}

		$reserved = mysql_num_rows(mysql_query("SELECT * FROM reservations WHERE reserve_user_id=" . USER_ID));
		$messages = mysql_num_rows(mysql_query("SELECT * FROM messages WHERE message_to_id=" . USER_ID));

		$bug_form = "
			<form name='bugtracker'>
				<input type='hidden' name='request' value='$request' />
				<input type='hidden' name='username' value='" . USER_NAME . "' />
				<input type='hidden' name='user_realname' value='$user_realname' />
				<input type='hidden' name='user_id' value='" . USER_ID . "' />
				<input type='hidden' name='card_number' value='" . USER_CARD_NUMBER . "' />
				<input type='hidden' name='email' value='" . USER_EMAIL . "' />
				<input type='hidden' name='privs' value='$privs' />
				<input type='hidden' name='language' value='" . USER_LANG . "' />
				<input type='hidden' name='template' value='" . USER_TEMPLATE . "' />
				<input type='hidden' name='cast' value='" . USER_CAST . "' />
				<input type='hidden' name='location' value='" . USER_LOCATION . "' />
				<input type='hidden' name='in_groups' value='$in_groups' />
				<input type='hidden' name='manager_of' value='$manager_of' />
				<input type='hidden' name='borrowed' value='$borrowed' />
				<input type='hidden' name='expired' value='$expired' />
				<input type='hidden' name='reserved' value='$reserved' />
				<input type='hidden' name='messages' value='$messages' />
			</form>
		";
		$content = str_replace("%bugtracker%", $bug_form, $content);
	}
	else {
		$content = str_replace("%bugtracker%", "", $content);
	}
}

//<-----
//	layout_print_part:	Print the specified part as defined by the parameter.
//----->
function layout_print_part($filename, $buttons = array())
{
	$content = layout_read_template($filename);

	if (!is_array($buttons)) {
		$buttons = array($buttons);
	}

	foreach ($buttons as $button)
	{
		//switch used for clarity, could be removed
		switch ($button) {

			case 1:
				layout_part_tag_replace(1, $content, has_category_priv(1));
				break;

			case 2:
				layout_part_tag_replace(2, $content, has_category_priv(2));
				break;

			case 3:
				layout_part_tag_replace(3, $content, has_category_priv(3));
				break;

			case 4:
				layout_part_tag_replace(4, $content, has_category_priv(4));
				break;

			case 5:
				layout_part_tag_replace(5, $content, has_category_priv(5));
				break;

			case 6:
				layout_part_tag_replace(6, $content, has_category_priv(6));
				break;

			default:
				break;
		}
	}

	$img_path = "templates/" . USER_TEMPLATE . "/img/";

	$content = str_replace("%img_template_path%", $img_path, $content);
	$content = str_replace('%location%', layout_get_location(), $content);

	print $content;
}

//<-----
//	layout_print_footer:	Print the footer from the template.
//							Might contain other actions in the future.
//----->
function layout_print_footer($filename)
{
	layout_print_part($filename);
}

//<-----
//	layout_header:	Reads the layout config for the template in question and
//					prints the layout parts accordingly.
//----->
function layout_header($args = array())
{
	global $page_config;
	global $layout_config;
	global $PageID;

	foreach ($layout_config as $section => $variables)
	{
		switch ($section)
		{
			case 'header':
				layout_print_header($args);
				break;

			case 'submenu':
				$submenu_vars = layout_get_submenu($PageID);
				layout_print_submenu(
					$submenu_vars['menu_href'],
					$submenu_vars['menu_text'],
					$submenu_vars['menu_target']
				);
				break;

			case 'content':
				break 2;

			default:
				$buttons = array();

				if (array_key_exists('buttons', $layout_config[$section]))  {
					$buttons = $layout_config[$section]['buttons'];
				}

				layout_print_part($variables['filename'], $buttons);
				break;
		}
	}
}

//<-----
//	layout_header:	Reads the layout config for the template in question and
//					prints the layout parts accordingly.
//----->
function layout_footer()
{
	global $page_config;
	global $layout_config;
	global $PageID;

	$config_found = FALSE;

	foreach ($layout_config as $section => $variables)
	{
		if (!$config_found && $section != 'content') {
			continue;
		}

		if ($section == 'content') {
			$config_found = TRUE;
			continue;
		}

		switch ($section)
		{
			case 'submenu':
				$submenu_vars = layout_get_submenu($PageID);
				layout_print_submenu(
					$submenu_vars['menu_href'],
					$submenu_vars['menu_text'],
					$submenu_vars['menu_target']
				);
				break;

			case 'footer':
				layout_print_footer($variables['filename']);
				break 2; //footer must be last item

			default:
				$buttons = array();

				if (array_key_exists('buttons', $layout_config[$section]))  {
					$buttons = $layout_config[$section]['buttons'];
				}

				layout_print_part($variables['filename'], $buttons);
				break;
		}
	}
}

//<-----
//	layout_imgW: 	Returns the width of image $imagePath (could be merged)
//					If $imagePath not found: tries template path
//----->
function layout_imgW($imagePath, $returnHandle = 0)
{
	if (!file_exists($imagePath))
		$imagePath = layout_get_img_path($imagePath);

	if (!file_exists($imagePath))
		return 0;
	else
	{
		$imageSize = GetImageSize($imagePath);
	
		//$returnHandle: 0 = width, 1 = height
		return $imageSize[$returnHandle];
	}
}

//<-----
//	layout_imgH: 	Returns the height of image $imagePath
//					If $imagePath not found: tries template path
//----->
function layout_imgH($imagePath)
{
	return layout_imgW($imagePath, 1);
}


//<-----
//	layout_get_img_path: 	Returns the template path of image $imgName
//----->
function layout_get_img_path($imgName)
{
	$imgPath = "templates/" . USER_TEMPLATE . "/img/" . $imgName;

	if (!file_exists($imgPath)) {
		$imgPath = "templates/default/img/" . $imgName;
	}

	return $imgPath;
}

//<-----
//	layout_insertImg: Inserts <img src ... > code for image $imgName
//----->
function layout_insertImg($imgName, $alt = "", $class = "", $width = 0, $height = 0, $align = '')
{
	$imgPath = layout_get_img_path($imgName);
	
	if ($width == 0) {
		$width = layout_imgW($imgPath);
	}
	if ($height == 0) {
		$height = layout_imgH($imgPath);
	}
	
	if ($align != '') {
		$align = ' align="' . $align . '"';
	}

	if ($class == '') {
		$class = 'default_image';
	}

	$src = "<img src='" . $imgPath . "' width=" . $width . " height=" . $height . " alt='" . $alt . "' class='" . $class . "' border='0'$align>";

	return $src;
}

function layout_blankImg($width = 1, $height = 1)
{
	return layout_insertImg('_blank.png', '', '', $width, $height);
}

//<-----
//	void layout_table_td([string content], [string options])
//
//	Prints a cell in a table, an empty cell by default.
//----->
function layout_table_td($content = '&nbsp;', $options = '', $space = TRUE)
{
	$params = '';
	$default_attributes = array(
								'align' => 'left',
								'valign' => 'middle',
								'class' => 'default_td'
								);

	foreach ($default_attributes as $attribute => $value)
	{
		//check whether any of the default attributes are overridden by the options argument
		$pos = strpos(strtolower(' ' . $options), ' ' . $attribute . '=');
		if (!is_int($pos)) {
			$params .= ' ' . $attribute . '="' . $value . '"';
		}
	}

	$options = ($options == '') ? $options : ' ' . $options;

	$cols = 0;
	$startpos = strpos(strtolower($options), 'colspan=');
	if ($space && is_int($startpos)) //if colspan is found among $options
	{
		//compensate for extra &nbsp; columns created by autospacer funktion

		$length = strpos(strtolower($options), ' ', $startpos); //find end of colspan="n" string portion

		if ($length === FALSE) { //if colspan is last option
			$length = strlen($options) - $startpos; //take until end of $options
		}

		$colspan = substr($options, $startpos, $length); //extract the colspan option
		ereg ('[^0-9]*([0-9]+)[^0-9]*', $colspan, $regs); //place all integers from $colspan into array $regs
		array_shift($regs); //remove $regs[0] (copy of string $colspan)
		$cols = implode($regs); //take all integers found and merge them into original value
		$colspan = 'colspan="' . ($cols * 2 - 1) . '"'; //add the extra space-columns

		$options = substr_replace($options, $colspan, $startpos, $length); //replace it
	}

	//print all $options + that of $params not inclued in $options
	print "\t<td" . $params . $options . ">" . $content . "</td>\n";

	return (($cols - 1) <= 0) ? 0 : $cols - 1; //number of EXTRA cells covered due to colspan
}

//<-----
//	void layout_table_header(array headers, [array|string options], [string|int forall], [string table_options])
//
//	Prints the header row of a table
//	headers: The values to put as column headers
//	[options: If not default options (alignment, etc). A string can only be used w/ forall]
//	[forAll ('forall' | 1): if $options is string and should be applied to all]
//	[table_options: a string containing optional attributes for the main table]
//----->
function layout_table_header($headers, $options = array(), $forAll = '', $table_options = '', $space = TRUE)
{
	$maxLen = sizeof($headers);
	$blankPath = layout_get_img_path('_blank.png');

	if (!is_array($options)) {
		$options = array($options);
	}

	if (sizeof($options) != $maxLen)
	{
		if (strtolower($forAll) == 'forall' || $forAll == 1)
		{
			$fill = $options[0];
			$options = array_fill(0, $maxLen, $fill);
		}
		else
		{
			$fill = '';
			$options = $options + array_fill(sizeof($options), $maxLen - sizeof($options), $fill);
		}
	}

	//start check for overriding attributes in table_options argument
	$params = '';
	$default_attributes = array(
								'border' => '0',
								'cellspacing' => '0',
								'cellpadding' => '3',
								);

	foreach ($default_attributes as $attribute => $value)
	{
		//check whether any of the default attributes are overridden by the table_options argument
		$pos = strpos(strtolower(' ' . $table_options), ' ' . $attribute . '=');
		if (!is_integer($pos)) {
    		$params .= ' ' . $attribute . '="' . $value . '"';
		}
	}

	$table_options = ($table_options == '') ? $table_options : ' ' . $table_options;

	print "<table" . $params . $table_options . ">\n";
	print "<tr>\n";
	layout_table_td('<img src="' . $blankPath . '" width="13">', 'class="tableTopLeft" width="13"');

	$cols = 0; //the colspan counter, colspan value returned by layout_table_td
	for ($i = 0; $i < $maxLen; $i++)
	{
		$addOption = 'class="tableTop" height="35" valign="bottom" nowrap';

		if ($options[$i] != '') {
			$addOption .= ' ' . $options[$i];
		}

		$cols += layout_table_td($headers[$i], $addOption, $space);

		if ($space && $i < ($maxLen - 1)) {
			layout_table_td('&nbsp;', $addOption);
		}
	}

	layout_table_td('<img src="' . $blankPath . '" width="10">', 'class="tableTopRight" width="10"');
	print "</tr>\n";

	if ($space) {
		$colspan = sizeof($headers) + $cols;
	}
	else {
		$colspan = sizeof($headers);
	}

	layout_table_footer('', $colspan); //store the number of headers in memory of table_footer
}

//<-----
//	void layout_table_footer(array headers)
//
//	Prints the table footer according to the number of elements in the header
//----->
function layout_table_footer($footer = '', $size = -1)
{
	static $maxLen;

	if ($size != -1) //just memorize
	{
		$maxLen = $size;
	}
	else //use the memorized value
	{
		print "<tr>\n";
		layout_table_td('&nbsp;', 'class="tableBottomLeft"');
		layout_table_td('&nbsp;', 'class="tableBottom" height="10" colspan="' . $maxLen . '"');
		layout_table_td('&nbsp;', 'class="tableBottomRight"');
		print "</tr>\n";

		if ($footer != '')
		{
			print "<tr>\n<td height='6'></td>\n</tr>\n";
			print "<tr>\n";
			layout_table_td('&nbsp;');
			layout_table_td($footer, 'align="right" nowrap colspan="' . $maxLen . '"');
			layout_table_td('&nbsp;');
			print "</tr>\n";
		}

		print "</table>\n";

	}
}

//<-----
//	void layout_table_row(string content, [int|string class], [array options], [int|string forall])
//
//	Prints a row in a table. If class is not defined or set as 'auto', the function
//	will automatically alternate between two CSS classes for the row with different
//	color settings. Otherwise the same options as for layout_print_header().
//----->
function layout_table_row($content, $class = 'auto', $options = array(), $forAll = '', $space = TRUE)
{
	static	$class_memory,
			$class_array_memory;

	if ($class == 'auto') {
		$class_memory = ($class_memory == 2) ? 1 : 2;
		$class = $class_memory;
	}
	elseif (is_array($class)) {

		if (empty($class_array_memory)) {
			$class_array_memory = 0;
		}

		$tmp = $class[$class_array_memory];

		$class_array_memory++;
		if ($class_array_memory >= sizeof($class)) {
			$class_array_memory = 0;
		}

		$class = $tmp;
	}
	else {
		$class_memory = $class;
	}

	$maxLen = sizeof($content);

	if (!is_array($options)) {
		$options = array($options);
	}

	if (sizeof($options) != $maxLen)
	{
		if (strtolower($forAll) == 'forall' || $forAll == 1)
		{
			$fill = $options[0];
			$options = array_fill(0, $maxLen, $fill);
		}
		else
		{
			$fill = '';
			$options = $options + array_fill(sizeof($options), $maxLen - sizeof($options), $fill);
		}
	}

	print "<tr>\n";
	layout_table_td('&nbsp;', 'class="tableLeft0' . $class . '"');

	for ($i = 0; $i < $maxLen; $i++)
	{
		$addOption = 'class="tableContent0' . $class . '"';

		if ($options[$i] != '') {
			$addOption .= ' ' . $options[$i];
		}

		layout_table_td($content[$i], $addOption);

		if ($space && $i < ($maxLen - 1)) {
			layout_table_td('&nbsp;', $addOption);
		}
	}

	layout_table_td('&nbsp;', 'class="tableRight0' . $class . '"');
	print "</tr>\n";
}

function layout_table_sort ($url_start, $columns = array())
{
	global $langText;
	$headers = array();

	if (!empty ($_REQUEST['order_by']))
	{
		foreach ($columns as $current)
		{
			$sortURL = '<table border="0" cellspacing="0" cellpadding="0"><tr><td><a href="' . $url_start;

			if ($_REQUEST['order_by'] != $current)
			{
				$sortURL .= "&order_by=$current&order_dir=ASC";
				$sortImg = '';
			}
			elseif ($_REQUEST['order_dir'] == 'ASC')
			{
				$sortURL .= "&order_by=$current&order_dir=DESC";
				$sortImg = layout_insertImg ('icon_sort_asc.png');
			}
			elseif ($_REQUEST['order_dir'] == "DESC")
			{
				$sortURL .= "&order_by=$current&order_dir=ASC";
				$sortImg = layout_insertImg ("icon_sort_desc.png");
			}

			$sortURL .= '">' . $langText[strtoupper ($current)] . '</a>';
			$sortURL .= "</td><td>&nbsp;$sortImg</td></tr></table>";

			array_push($headers, $sortURL);
		}
	}

	return $headers;
}

//<-----
//
//----->
function layout_wizard_start($help_title = '', $help_text = '')
{
	global $langText;

	if ($help_title == '') {
		$help_title = (isset($langText['HELP_DESC'])) ? $langText['HELP_DESC'] : $help_title;
	}

	if ($help_text == '') {
		$help_text = (isset($langText['HELP_TEXT'])) ? $langText['HELP_TEXT'] : $help_text;
	}

	layout_wizard_stop($help_title, $help_text, 'memorize');
}

//<-----
//
//----->
function layout_wizard_stop($help_title = '', $help_text = '', $action = '')
{
	static	$title,
			$text;

	if ($action == 'memorize')
	{
		$help_text = '<font class="helptext">' . $help_text . '</font>';
		$help_title = '<font class="helptitle">' . $help_title . '</font>';

		print '
				<table cellspacing="0" cellpadding="0" border="0">
					<tr>
						<td align="left" valign="top">
		';

		$title = $help_title;
		$text = $help_text;
	}
	else
	{
		if ($help_title != '' && $help_text != '')
		{
			$title = $help_title;
			$text = $help_text;
		}

		print '
				</td>
				<td width="40">&nbsp;</td>
				<td align="left" valign="top">
		';

		?>

		<table cellspacing="0" width="280">
			<tr>
				<td align="left" valign="top" class="wizard">
					<?php echo $title ?>
					<br /><br />
					<?php echo $text ?>
				</td>
			</tr>
		</table>

		<?php

		//layout_table_header(array($title), array(), 0, 'width="80%"');
		//layout_table_row(array($text), 1);
		//layout_table_footer();

		print '
						</td>
						<td width="20">&nbsp;</td>
					</tr>
				</table>
		';
	}
}

//<-----
//
//----->
function layout_page_title($text = '', $class = '', $once_only = TRUE, $auto_hide = TRUE)
{
	global	$config,
			$langText;

	//layout_page_title() can be called only once per page
	static $title_count;
	$title_count++;

	if ($class != '')
	{
		switch (strtolower($class))
		{
			case 'error':
				$classStart = '<span class="red" id="title_message">';
				$classEnd = '</span>';
				break;

			case 'ok':
				$classStart = '<span class="green" id="title_message">';
				$classEnd = '</span>';
				break;

			default:
				$classStart = '<span class="' . $class . '" id="title_message">';
				$classEnd = '</span>';
				break;
		}
	}
	else
	{
		$classStart = '';
		$classEnd = '';
	}

	if ($class != '' && $auto_hide)
	{
		print "
		<script language='JavaScript'><!--// written with the help of examples at www.experts-exchange.com

			var css;

			if (document.getElementById) {
				css = 'cssRules';
			}
			if (document.all) {
				css = 'rules';
			}

			for (var i = 0; i < document.styleSheets[0][css].length; i++)
			{
				if (document.styleSheets[0][css][i].selectorText == '.$class')
				{
					var colorRegex = /rgb\((\d+),\s*(\d+),\s*(\d+)\)/;
					colorRegex.exec(document.styleSheets[0][css][i].style.color);
					var r = parseInt(RegExp.$1) + 1;
					var g = parseInt(RegExp.$2) + 1;
					var b = parseInt(RegExp.$3) + 1;
				}
			}

			function dec2hex(dec)
			{
				var hexChars = '0123456789ABCDEF';

				if (dec < 0) {
					return '00';
				}
				else if (dec > 255) {
					return 'FF';
				}

				var i = Math.floor(dec / 16)
				var j = dec % 16

				return hexChars.charAt(i) + hexChars.charAt(j)
			}

			function hex2dec(hex)
			{
				return parseInt('0x' + hex, 16);
			}

			function message_style(property, value)
			{
				if (document.getElementById) {
					document.getElementById('title_message').style[property] = value;
				}
				else if (document.all) {
					document.all['title_message'].style[property] = value;
				}
				else if (document.layers) {
					document.layers['title_message'][property] = value;
				}
			}

			function message_fade()
			{
				if (r < 255 || g < 255 || b < 255)
				{
					r += 1 + (255 - r) / 10;
					g += 1 + (255 - g) / 10;
					b += 1 + (255 - b) / 10;

					color = dec2hex(r) + dec2hex(g) + dec2hex(b);
					message_style('color', '#' + color);

					setTimeout('message_fade()', 50);
				}
				else {
					message_style('visibility', 'hidden');
				}
			}

			hide = setTimeout('message_fade()', " . ($config['redirect_timeout'] * 1000) . ");

		//--></script>
		";
	}

	if (!($once_only && $title_count > 1))
	{
		$bottom = '<br />';

		if ($text == '' || trim(strip_tags($text)) == '') {
			$text = (isset($langText['HEADER'])) ? $langText['HEADER'] : '';
		}

		if (isset($langText['HEADER']) && $text != $langText['HEADER']) {
			$text = $langText['HEADER'] . $classStart . ' - ' . $text . $classEnd;
		}
		elseif (!isset($langText['HEADER'])) {
			$text = $classStart . $text . $classEnd;
			$bottom = '';
		}

		print '<br /><font class="pagetitle">' . $text . '</font><br /><br />' . $bottom;
	}
}

//<-----
//
//----->
function layout_page_info($text, $class = '', $once_only = FALSE)
{
	//DO NOT USE DIRECTLY AFTER layout_page_title(),
	//UNLESS ALWAYS VISIBLE (creates layout "jump")

	static $info_count;

	if ($once_only) {
		$info_count++;
	}

	if (!($once_only && $info_count > 1))
	{
		switch ($class)
		{
			case 'error':
				$class = 'red';
				break;

			case 'ok':
				$class = 'green';
				break;
		}

		$class = ($class != '') ? ' class="' . $class . '"' : '';
		print "<h4$class>$text</h4>";
	}
}

//<-----
//
//----->
function layout_management_panel($args, $split = FALSE, $maxrows = 4)
{
	echo '<table cellpadding="0" cellspacing="0" border="0" width="100%">';

	//remove all non-priv rows
	foreach ($args as $key => $row)
	{
		if (!isset($row['page_id']) || !has_page_priv($row['page_id'])) {
			unset($args[$key]);
		}
		if (isset($row['priv']) && !check_priv($row['priv'])) {
			unset($args[$key]);
		}
	}

	$i = 0;
	foreach ($args as $row)
	{
		if(!empty($row))
		{
			//if > 4 elements, switch to 2 columns
			print ((!$split && sizeof($args) <= $maxrows) || $i % 2 == 0) ? "\n<tr>" : '';

			if(stristr($row['href'], "?"))
			{
				$href = $row['href'];
			}
			else
			{
				$href = $row['href'].'?id='.$_REQUEST['id'].'&auth='.$_REQUEST['auth'];
			}


			print	'
				<td align="left" valign="middle" width="50%">

					<table border="0" cellspacing="0" cellpadding="12">
						<tr>
							<td align="left" valign="middle">
								<a href="'.$href.'">' . layout_insertImg($row['img']) . '</a>
							</td>
							<td align="left" valign="middle">
								' . $row['text'] . '
							</td>
						</tr>
					</table>

				</td>
			';

			print ((!$split && sizeof($args) <= $maxrows) || $i % 2 != 0) ? '</tr>' : "\n<td>&nbsp;</td>\n";

			$i++;
		}
	}

	echo '</table>';
}

//<-----
//
//----->
function receipt_text($type = '', $language = USER_LANG)
{
	global	$langText,
			$config;

	$header = '';
	$footer = '';

	$row = mysql_fetch_array(mysql_query("SELECT * FROM locations WHERE location_id=" . USER_LOCATION));

	if ($language != USER_LANG) {
		$langTmp = $langText;
		parseLang($language);
	}

	$header = SOFTWARE;

	if ($type == 'user_info') {
		$footer .= $langText['PLEASE_CHANGE_PASSWORD'] . '[nl][nl]';
	}

	$footer = 	SOFTWARE . ' ' . VERSION . '[nl]' .
				VENDOR . '[nl]';

	if ($type != 'borrow' && $type != 'return') {
		$footer .= date($config['dateformat'] . '  ' . $config['timeformat']) . '[nl]';
	}

	$footer .= '[nl]';

	if(!empty($row['location_name'])) $footer .= $row['location_name'] . '[nl]';
	if(!empty($row['location_postal_address'])) $footer .= $row['location_postal_address'] . '[nl]';
	if(!empty($row['location_postal_code']) && !empty($row['location_town'])) $footer .= $row['location_postal_code'] . ' ' . $row['location_town'] . '[nl]';
	$footer .= '[nl]';
	if(!empty($row['location_phone'])) $footer .= $langText['PHONE'] . ': ' . $row['location_phone'] . '[nl]';
	if(!empty($row['location_fax'])) $footer .= $langText['FAX'] . ': ' . $row['location_fax'] . '[nl]';
	if(!empty($row['location_notes'])) $footer .= $row['location_notes'] . '[nl]';

	if ($language != USER_LANG) {
		$langText = $langTmp;
	}

	return array($header, $footer);
}
?>
